package com.caimei.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.caimei.model.ResponseJson;
import com.caimei.service.OrderSubmitService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/4/25
 */
@Api(tags = "订单结算")
@Slf4j
@RestController
@RequiredArgsConstructor
@RequestMapping("/order")
public class OrderSubmitApi {
    private final OrderSubmitService orderSubmitService;

    /**
     * 商品结算
     */
    @ApiOperation("商品结算")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "cartIds", value = "购物车ID串用逗号隔开", required = false),
            @ApiImplicitParam(name = "productId", value = "立即购买商品ID", required = false),
            @ApiImplicitParam(name = "productCount", value = "立即购买商品数量", required = false),
            @ApiImplicitParam(name = "heUserId", value = "分销者用户id", required = false)
    })
    @GetMapping("/confirm")
    public ResponseJson<Map<String, Object>> orderConfirm(String cartIds, Integer productId, Integer productCount, Integer heUserId) {
        return orderSubmitService.orderConfirm(cartIds, productId, productCount, heUserId);
    }

    /**
     * 提交订单
     *
     * @param params 参数格式：{
     *               *   "userId": 10708,        //机构用户ID
     *               *   "cartType":3,               //购买类型:（1购物车提交[对应表cm_cart]，2直接购买提交）
     *               *   "addressId": 2732,          //地址ID
     *               *   "orderInfo": [
     *               *                   { "shopId":1001,    // 供应商ID
     *               *                     "note":备注,
     *               *                     "productInfo":[   // 商品id，数量,分销者userId
     *               *                         {"productId": 2789, "productNum": 1,"heUserId":1010},
     *               *                         {"productId": 2789, "productNum": 1,"heUserId":0}
     *               *                      ]
     *               *                   },
     *               *                   {多个供应商商品数据结构同上}
     *               *                ]
     *               *
     *               *   "payInfo": {               // 订单信息
     *               *       "orderShouldPayFee": 609.11
     *               *   }
     * @return code: -1=(用户账户异常,数据异常,操作异常等)，
     * 1提交成功（支付完成）,
     * 2提交成功(未支付)，
     * code为1和2的情况下出现 data{orderNo:订单号，orderID:订单ID,payTotalFee:订单金额，orderMark：订单标识}
     */
    @ApiOperation("提交订单")
    @PostMapping("/submit")
    public ResponseJson<Map<String, String>> orderSubmit(@RequestBody String params) {
        if (StringUtils.isBlank(params)) {
            return ResponseJson.error("数据异常", null);
        }
        Integer userId = null;
        Integer cartType = null;
        Integer addressId = null;
        List<Map<String, Object>> orderInfo = null;
        Map<String, Object> payInfo = null;
        Map<String, Object> orderInvoice = null;
        // 校验传入参数的正确性
        try {
            JSONObject paramsMap = JSON.parseObject(params);
            if (null == paramsMap) {
                return ResponseJson.error("数据异常,参数不能为空", null);
            }
            userId = paramsMap.getInteger("userId");
            if (null == userId) {
                return ResponseJson.error("机构用户Id异常", null);
            }
            cartType = paramsMap.getInteger("cartType");
            if (null == cartType) {
                return ResponseJson.error("购买类型异常", null);
            }
            addressId = paramsMap.getInteger("addressId");
            if (null == addressId) {
                return ResponseJson.error("地址数据异常", null);
            }
            orderInfo = (List<Map<String, Object>>) paramsMap.get("orderInfo");
            if (null == orderInfo) {
                return ResponseJson.error("订单商品数据异常", null);
            }
            payInfo = paramsMap.getJSONObject("payInfo");
            if (null == payInfo) {
                return ResponseJson.error("订单金额数据异常", null);
            }
        } catch (Exception e) {
            log.error("<<<<< 系统异常 >>>>>" + e);
            return ResponseJson.error("数据异常", null);
        }

        log.info("****** 提交订单参数：【机构自己下单】：" + params);
        return orderSubmitService.orderSubmit(userId, cartType, addressId, orderInfo, payInfo);
    }
}
